"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const fs_1 = require("fs");
const path = require("path");
const heimdall_1 = require("./lib/heimdall");
const output_patcher_1 = require("./lib/output-patcher");
const plugin_1 = require("./lib/plugin");
const tree_diff_1 = require("./lib/tree-diff");
// tslint:disable-next-line:no-var-requires
const symlinkOrCopySync = require('symlink-or-copy').sync;
// tslint:disable-next-line:no-var-requires
const nodeModulesPath = require('node-modules-path');
const deref = typeof fs_1.copyFileSync === 'function' ?
    (srcPath, destPath) => {
        try {
            fs_1.unlinkSync(destPath);
        }
        catch (e) {
            if (e.code !== 'ENOENT') {
                throw e;
            }
        }
        fs_1.copyFileSync(srcPath, destPath, fs_1.constants.COPYFILE_EXCL);
    } :
    (srcPath, destPath) => {
        const content = fs_1.readFileSync(srcPath);
        fs_1.writeFileSync(destPath, content);
    };
module.exports = class Rollup extends plugin_1.default {
    constructor(node, options) {
        super([node], {
            annotation: options.annotation,
            name: options.name,
            persistentOutput: true,
        });
        this.rollupOptions = options.rollup;
        this._lastBundle = null;
        this._output = null;
        this.lastTree = tree_diff_1.treeFromEntries([]);
        this.linkedModules = false;
        this.cache = options.cache === undefined ? true : options.cache;
        if (options.nodeModulesPath !== undefined && !path.isAbsolute(options.nodeModulesPath)) {
            throw new Error(`nodeModulesPath must be fully qualified and you passed a relative path`);
        }
        this.nodeModulesPath = options.nodeModulesPath || nodeModulesPath(process.cwd());
    }
    build() {
        const lastTree = this.lastTree;
        const linkedModules = this.linkedModules;
        if (!linkedModules) {
            symlinkOrCopySync(this.nodeModulesPath, `${this.cachePath}/node_modules`);
            this.linkedModules = true;
        }
        const newTree = this.lastTree = tree_diff_1.treeFromPath(this.inputPaths[0]);
        const patches = lastTree.calculatePatch(newTree);
        patches.forEach((change) => {
            const op = change[0];
            const relativePath = change[1];
            switch (op) {
                case 'mkdir':
                    fs_1.mkdirSync(`${this.cachePath}/${relativePath}`);
                    break;
                case 'unlink':
                    fs_1.unlinkSync(`${this.cachePath}/${relativePath}`);
                    break;
                case 'rmdir':
                    fs_1.rmdirSync(`${this.cachePath}/${relativePath}`);
                    break;
                case 'create':
                    deref(`${this.inputPaths[0]}/${relativePath}`, `${this.cachePath}/${relativePath}`);
                    break;
                case 'change':
                    deref(`${this.inputPaths[0]}/${relativePath}`, `${this.cachePath}/${relativePath}`);
                    break;
            }
        });
        // If this a noop post initial build, just bail out
        if (this._lastBundle && patches.length === 0) {
            return;
        }
        const options = this._loadOptions();
        options.input = this.cachePath + '/' + options.input;
        return heimdall_1.instrument('rollup', () => {
            return require('rollup').rollup(options)
                .then((bundle) => {
                if (this.cache) {
                    this._lastBundle = bundle;
                }
                return this._buildTargets(bundle, options);
            });
        });
    }
    _loadOptions() {
        // TODO: support rollup config files
        const options = Object.assign({
            cache: this._lastBundle,
        }, this.rollupOptions);
        return options;
    }
    _targetsFor(options) {
        return Array.isArray(options.output) ? options.output : [options.output];
    }
    _buildTargets(bundle, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const output = this._getOutput();
            const targets = this._targetsFor(options);
            for (let i = 0; i < targets.length; i++) {
                yield this._buildTarget(bundle, targets[i], output);
            }
            output.patch();
        });
    }
    _buildTarget(bundle, options, output) {
        return __awaiter(this, void 0, void 0, function* () {
            // const { dest, sourceMap, sourceMapFile } = options;
            const file = options.file;
            const sourcemap = options.sourcemap;
            const sourcemapFile = options.sourcemapFile;
            // ensures "file" entry and relative "sources" entries
            // are correct in the source map.
            if (sourcemapFile) {
                options.sourcemapFile = this.cachePath + '/' + sourcemapFile;
            }
            else {
                options.sourcemapFile = this.cachePath + '/' + file;
            }
            const result = yield bundle.generate(Object.assign({}, options, {
                sourcemap: !!sourcemap,
            }));
            let code = result.code;
            const map = result.map;
            if (sourcemap && map !== null) {
                let url;
                if (sourcemap === 'inline') {
                    url = map.toUrl();
                }
                else {
                    url = this._addSourceMap(map, file, output);
                }
                code += '//# sourceMap';
                code += `pingURL=${url}`;
            }
            output.add(file, code);
        });
    }
    _addSourceMap(map, relativePath, output) {
        const url = path.basename(relativePath) + '.map';
        output.add(relativePath + '.map', map.toString());
        return url;
    }
    _getOutput() {
        let output = this._output;
        if (!output) {
            output = this._output = new output_patcher_1.default(this.outputPath, heimdall_1.logger);
        }
        return output;
    }
};
//# sourceMappingURL=index.js.map